@@ -21,10 +21,11 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-import types
+from types import UnicodeType, ListType, TupleType
 from copy import deepcopy
 
-from imdb.utils import analyze_title, build_title, normalizeTitle, _Container
+from imdb.utils import analyze_title, build_title, normalizeTitle, \
+                        flatten, _Container
 
 
 class Movie(_Container):
@@ -106,6 +107,7 @@ class Movie(_Container):
                 'tv guests': 'guests',
                 'notable tv guest appearances': 'guests',
                 'episodes cast': 'guests',
+                'episodes number': 'number of episodes',
                 'amazon review': 'amazon reviews',
                 'merchandising': 'merchandising links',
                 'merchandise': 'merchandising links',
@@ -152,13 +154,33 @@ class Movie(_Container):
 
     def _additional_keys(self):
         """Valid keys to append to the data.keys() list."""
+        addkeys = []
         if self.data.has_key('title'):
-            return ['canonical title', 'long imdb title',
-                    'long imdb canonical title']
-        return []
+            addkeys += ['canonical title', 'long imdb title',
+                        'long imdb canonical title']
+        if self.data.has_key('episode of'):
+            addkeys += ['long imdb episode title', 'series title',
+                        'canonical series title', 'episode title',
+                        'canonical episode title']
+        return addkeys
 
     def _getitem(self, key):
         """Handle special keys."""
+        if self.data.has_key('episode of'):
+            if key == 'long imdb episode title':
+                return build_title(self.data, canonical=0)
+            elif key == 'series title':
+                ser_title = self.data['episode of'].get('canonical title') or \
+                            self.data['episode of']['title']
+                return normalizeTitle(ser_title)
+            elif key == 'canonical series title':
+                ser_title = self.data['episode of'].get('canonical title') or \
+                            self.data['episode of']['title']
+                return ser_title
+            elif key == 'episode title':
+                return normalizeTitle(self.data.get('title', u''))
+            elif key == 'canonical episode title':
+                return self.data.get('title', u'')
         if self.data.has_key('title'):
             if key == 'title':
                 return normalizeTitle(self.data['title'])
@@ -170,6 +192,10 @@ class Movie(_Container):
                 return build_title(self.data, canonical=1)
         return None
 
+    def getID(self):
+        """Return the movieID."""
+        return self.movieID
+
     def __nonzero__(self):
         """The Movie is "false" if the self.data does not contain a title."""
         # XXX: check the title and the movieID?
@@ -180,6 +206,7 @@ class Movie(_Container):
         """Return true if this and the compared object have the same
         long imdb title and/or movieID.
         """
+        # XXX: obsolete?
         if not isinstance(other, self.__class__): return 0
         if self.data.has_key('title') and \
                 other.data.has_key('title') and \
@@ -196,11 +223,9 @@ class Movie(_Container):
         from Person import Person
         if not isinstance(item, Person):
             return 0
-        for i in self.data.values():
-            if type(i) in (types.ListType, types.TupleType):
-                for j in i:
-                    if isinstance(j, Person) and item.isSamePerson(j):
-                        return 1
+        for p in flatten(self.data, yieldDictKeys=1, scalar=Person):
+            if item.isSame(p):
+                return 1
         return 0
 
     def __deepcopy__(self, memo):
@@ -218,9 +243,13 @@ class Movie(_Container):
     def __repr__(self):
         """String representation of a Movie object."""
         # XXX: add also currentRole and notes, if present?
-        r = '<Movie id:%s[%s] title:"%s">' % (self.movieID, self.accessSystem,
-                                        self.get('long imdb canonical title'))
-        if type(r) is type(u''): r = r.encode('utf_8', 'replace')
+        if self.has_key('long imdb episode title'):
+            title = self.get('long imdb episode title')
+        else:
+            title = self.get('long imdb canonical title')
+        r = '<Movie id:%s[%s] title:_%s_>' % (self.movieID, self.accessSystem,
+                                                title)
+        if isinstance(r, UnicodeType): r = r.encode('utf_8', 'replace')
         return r
 
     def __str__(self):
@@ -231,7 +260,6 @@ class Movie(_Container):
         """Simply print the short title."""
         return self.get('title', u'')
 
-        
     def summary(self):
         """Return a string with a pretty-printed summary for the movie."""
         if not self: return u''
