@@ -21,10 +21,11 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
-import types
+from types import UnicodeType, ListType, TupleType
 from copy import deepcopy
 
-from imdb.utils import analyze_name, build_name, normalizeName, _Container
+from imdb.utils import analyze_name, build_name, normalizeName, \
+                        flatten, _Container
 
 
 class Person(_Container):
@@ -137,6 +138,10 @@ class Person(_Container):
                 return build_name(self.data, canonical=1)
         return None
 
+    def getID(self):
+        """Return the personID."""
+        return self.personID
+
     def __nonzero__(self):
         """The Person is "false" if the self.data does not contain a name."""
         # XXX: check the name and the personID?
@@ -148,14 +153,12 @@ class Person(_Container):
         from Movie import Movie
         if not isinstance(item, Movie):
             return 0
-        for i in self.data.values():
-            if type(i) in (types.ListType, types.TupleType):
-                for j in i:
-                    if isinstance(j, Movie) and item.isSameTitle(j):
-                        return 1
+        for m in flatten(self.data, yieldDictKeys=1, scalar=Movie):
+            if item.isSame(m):
+                return 1
         return 0
 
-    def isSamePerson(self, other):
+    def isSameName(self, other):
         """Return true if two persons have the same name and imdbIndex
         and/or personID.
         """
@@ -170,6 +173,7 @@ class Person(_Container):
                 self.personID and self.personID == other.personID:
             return 1
         return 0
+    isSamePerson = isSameName # XXX: just for compatibility.
 
     def __deepcopy__(self, memo):
         """Return a deep copy of a Person instance."""
@@ -187,9 +191,9 @@ class Person(_Container):
     def __repr__(self):
         """String representation of a Person object."""
         # XXX: add also currentRole and notes, if present?
-        r = '<Person id:%s[%s] name:"%s">' % (self.personID, self.accessSystem,
+        r = '<Person id:%s[%s] name:_%s_>' % (self.personID, self.accessSystem,
                                         self.get('long imdb canonical name'))
-        if type(r) is type(u''): r = r.encode('utf_8', 'replace')
+        if isinstance(r, UnicodeType): r = r.encode('utf_8', 'replace')
         return r
 
     def __str__(self):
