@@ -28,6 +28,8 @@ from urllib import FancyURLopener, quote_plus
 from codecs import lookup
 
 from imdb import IMDbBase
+from imdb.Movie import Movie
+from imdb.utils import analyze_title
 from imdb._exceptions import IMDbDataAccessError, IMDbParserError
 from movieParser import movie_parser, plot_parser, movie_awards_parser, \
                         taglines_parser, keywords_parser, \
@@ -40,12 +42,12 @@ from movieParser import movie_parser, plot_parser, movie_awards_parser, \
                         newsgrouprev_parser, misclinks_parser, \
                         soundclips_parser, videoclips_parser, news_parser, \
                         photosites_parser, amazonrev_parser, guests_parser, \
-                        business_parser, sales_parser
+                        business_parser, sales_parser, episodes_parser
 from searchMovieParser import search_movie_parser
 from personParser import maindetails_parser, bio_parser, \
                         otherworks_parser, person_awards_parser, \
                         person_officialsites_parser, publicity_parser, \
-                        agent_parser
+                        agent_parser, person_series_parser
 from searchPersonParser import search_person_parser
 from utils import ParserBase
 
@@ -69,6 +71,48 @@ class IMDbURLopener(FancyURLopener):
         #      so lets fake to be Mozilla.
         #      Wow!  I'm shocked by my total lack of ethic! <g>
         self.addheaders = [('User-agent', 'Mozilla/5.0')]
+        # This class is used also to perform "Exact Primary [Title|Name]"
+        # searches, and so by default the cookie is set.
+        c_header = 'id=%s; uu=%s' % (_cookie_id, _cookie_uu)
+        self.addheaders.append(('Cookie', c_header))
+
+    def retrieve_unicode(self, url):
+        """Retrieves the given URL, and returns a unicode string,
+        trying to guess the encoding of the data (assuming latin_1
+        by default)"""
+        encode = None
+        try:
+            uopener = self.open(url)
+            content = uopener.read()
+            server_encode = uopener.info().getparam('charset')
+            # look at the content-type HTML meta tag.
+            if server_encode is None and content:
+                first_bytes = content[:512]
+                begin_h = first_bytes.find('text/html; charset=')
+                if begin_h != -1:
+                    end_h = first_bytes[19+begin_h:].find('"')
+                    if end_h != -1:
+                        server_encode = first_bytes[19+begin_h:19+begin_h+end_h]
+            if server_encode:
+                try:
+                    if lookup(server_encode):
+                        encode = server_encode
+                except (LookupError, ValueError, TypeError):
+                    pass
+            uopener.close()
+            self.close()
+        except IOError, e:
+            raise IMDbDataAccessError, {'errcode': e.errno,
+                                        'errmsg': str(e.strerror),
+                                        'url': url,
+                                        'proxy': self.proxies.get('http', '')}
+        if encode is None:
+            encode = 'latin_1'
+            # The detection of the encoding is error prone...
+            import warnings
+            warnings.warn('Unable to detect the encoding of the retrieved '
+                        'page [%s]; falling back to default latin1.' % encode)
+        return unicode(content, encode, 'replace')
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
         raise IMDbDataAccessError, {'url': 'http:%s' % url,
@@ -142,7 +186,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def set_proxy(self, proxy):
         """Set the web proxy to use.
-        
+
         It should be a string like 'http://localhost:8080/'; if the
         string is empty, no proxy will be used.
         If set, the value of the environment variable HTTP_PROXY is
@@ -176,33 +220,7 @@ class IMDbHTTPAccessSystem(IMDbBase):
 
     def _retrieve(self, url):
         """Retrieve the given URL."""
-        encode = 'latin_1'
-        try:
-            uopener = self.urlOpener.open(url)
-            content = uopener.read()
-            server_encode = uopener.info().getparam('charset')
-            # look at the content-type HTML meta tag.
-            if server_encode is None and content:
-                first_bytes = content[:512]
-                begin_h = first_bytes.find('text/html; charset=')
-                if begin_h != -1:
-                    end_h = first_bytes[19+begin_h:].find('"')
-                    if end_h != -1:
-                        server_encode = first_bytes[19+begin_h:19+begin_h+end_h]
-            if server_encode:
-                try:
-                    if lookup(server_encode):
-                        encode = server_encode
-                except (LookupError, ValueError, TypeError):
-                    pass
-            uopener.close()
-            self.urlOpener.close()
-        except IOError, e:
-            raise IMDbDataAccessError, {'errcode': e.errno,
-                                        'errmsg': str(e.strerror),
-                                        'url': url,
-                                        'proxy': self.get_proxy()}
-        return unicode(content, encode, 'replace')
+        return self.urlOpener.retrieve_unicode(url)
 
     def _search_movie(self, title, results):
         # The URL of the query.
@@ -340,6 +358,21 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_movie % movieID + 'sales')
         return sales_parser.parse(cont)
 
+    def get_movie_episodes(self, movieID):
+        cont = self._retrieve(imdbURL_movie % movieID + 'episodes')
+        data_d = episodes_parser.parse(cont)
+        # set movie['episode of'] for every episode of the series.
+        if data_d.get('data', {}).has_key('episodes'):
+            nr_eps = 0
+            for season in data_d['data']['episodes'].values():
+                for episode in season.values():
+                    episode['episode of'].movieID = movieID
+                    nr_eps += 1
+            # Number of episodes.
+            if nr_eps:
+                data_d['data']['number of episodes'] = nr_eps
+        return data_d
+
     def _search_person(self, name, results):
         # The URL of the query.
         # XXX: To retrieve the complete results list:
@@ -386,4 +419,8 @@ class IMDbHTTPAccessSystem(IMDbBase):
         cont = self._retrieve(imdbURL_person % personID + 'news')
         return news_parser.parse(cont)
 
+    def get_person_episodes(self, personID):
+        cont = self._retrieve(imdbURL_person % personID + 'filmoseries')
+        return person_series_parser.parse(cont)
+
 
