@@ -26,6 +26,8 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
+import re
+from types import UnicodeType, StringType
 from urllib import unquote
 
 from imdb.Person import Person
@@ -35,9 +37,6 @@ from imdb._exceptions import IMDbParserError
 from utils import ParserBase
 
 
-_stypes = (type(u''), type(''))
-
-
 def strip_amps(theString):
     """Remove '&\S*' AT the end of a string.
     It's used to remove '& ' from strings like '(written by) & '.
@@ -124,6 +123,17 @@ class HTMLMovieParser(ParserBase):
         self._merge_next = 0
         # Counter for the billing position in credits.
         self._counter = 1
+        # For tv series.
+        self._in_h1 = 0
+        self._in_strong_title = 0
+        self._seen_title_br = 0
+        self._in_episode_title = 0
+        self._episode_title = u''
+        self._in_series_title = 0
+        self._series_title = u''
+        self._in_series_info = 0
+        self._series_info = ''
+        self._cur_id = ''
 
     def append_item(self, sect, data):
         """Append a new value in the given section of the dictionary."""
@@ -134,7 +144,7 @@ class HTMLMovieParser(ParserBase):
 
     def set_item(self, sect, data):
         """Put a single value (a string, normally) in the dictionary."""
-        if type(data) in _stypes:
+        if isinstance(data, (UnicodeType, StringType)):
             data = clear_text(data)
         self._movie_data[clear_text(sect)] = data
 
@@ -151,7 +161,24 @@ class HTMLMovieParser(ParserBase):
             d_title = analyze_title(self._title)
             for key, item in d_title.items():
                 self.set_item(key, item)
+            # XXX: what about _title_short for tv series' episodes?
             self._title_short = d_title.get('title', u'').lower()
+            if d_title.get('kind') in ('tv series', 'tv mini series'):
+                self._title_short = '"%s"' % self._title_short
+
+    def start_h1(self, attrs):
+        self._in_h1 = 1
+
+    def end_h1(self):
+        self._in_h1 = 0
+
+    def start_strong(self, attrs):
+        cls = self.get_attr_value(attrs, 'class')
+        if cls and cls.strip().lower() == 'title':
+            self._in_strong_title = 1
+
+    def end_strong(self):
+        self._in_strong_title = 0
 
     def start_table(self, attrs): pass
 
@@ -214,6 +241,10 @@ class HTMLMovieParser(ParserBase):
             self._isplotoutline = 0
             if self._plotoutline:
                 self.set_item('plot outline', self._plotoutline)
+        elif self._in_series_title:
+            ids = self.re_imdbID.findall(link)
+            if ids:
+                self._cur_id = ids[-1]
         elif link.startswith('http://pro.imdb.com'):
             self._is_movie_status = 0
         elif link.startswith('/titlebrowse?') and \
@@ -284,7 +315,7 @@ class HTMLMovieParser(ParserBase):
         elif self._is_cast_crew and self._current_section:
             if self._is_name:
                 self._name += '::'
-    
+
     def do_br(self, attrs):
         if self._is_company_cred:
             # Sometimes companies are separated by <br> tags.
@@ -329,6 +360,36 @@ class HTMLMovieParser(ParserBase):
             self._isplotoutline = 0
             if self._plotoutline:
                 self.set_item('plot outline', self._plotoutline)
+        elif self._in_series_title:
+            self._in_series_title = 0
+            st = self._series_title.strip()
+            if st and self._cur_id:
+                d_title = analyze_title(st, canonical=1)
+                m = Movie(movieID=str(self._cur_id), data=d_title,
+                            accessSystem='http')
+                self._movie_data['kind'] = 'episode'
+                self._movie_data['episode of'] = m
+            self._series_title = u''
+        elif self._in_series_info:
+            self._in_series_info = 0
+            si = ' '.join([x for x in self._series_info.split() if x])
+            if si:
+                aid = self.re_airdate.findall(si)
+                if aid and len(aid[0]) == 3:
+                    date, season, episode = aid[0]
+                    date = date.strip()
+                    try: season = int(season)
+                    except: pass
+                    try: episode = int(episode)
+                    except: pass
+                    if date and date != '????':
+                        self.set_item('original air date', date)
+                    # Handle also "episode 0".
+                    if season or type(season) is type(0):
+                        self.set_item('season', season)
+                    if episode or type(season) is type(0):
+                        self.set_item('episode', episode)
+            self._series_info = ''
         elif self._is_runtimes and self._runtimes:
             self._is_runtimes = 0
             rt = self._runtimes.replace(' min', '')
@@ -336,11 +397,33 @@ class HTMLMovieParser(ParserBase):
             episodes = re_episodes.findall(rt)
             if episodes:
                 rt = re_episodes.sub('', rt)
-                self.set_item('episodes', episodes[0])
+                episodes = episodes[0]
+                try: episodes = int(episodes)
+                except: pass
+                self.set_item('number of episodes', episodes)
             rl = [x.strip() for x in rt.split('/')]
             if rl: self.set_item('runtimes', rl)
         if self.mdparse:
             self.end_tr()
+        if self._in_h1 and self._in_strong_title:
+            self._seen_title_br = 1
+        else:
+            self._seen_title_br = 0
+
+    def start_small(self, attrs):
+        if self._in_h1 and self._in_strong_title and self._seen_title_br:
+            self._in_episode_title = 1
+
+    def end_small(self):
+        if self._in_episode_title:
+            self._in_episode_title = 0
+            self._episode_title = self._episode_title.strip()
+            if self._episode_title:
+                d_title = analyze_title(self._episode_title, canonical=1)
+                for key, item in d_title.items():
+                    if key == 'kind': continue
+                    self.set_item(key, item)
+                self._episode_title = u''
 
     def start_li(self, attrs): pass
 
@@ -378,8 +461,6 @@ class HTMLMovieParser(ParserBase):
             # The gold and grey stars; we're near the rating and number
             # of votes.
             self._is_rating = 1
-        # XXX: I've a report about alttex==self._movie_data.get('title')
-        #      in some situations, but I'm not sure.
         elif alttex == 'cover' or alttex == self._title_short:
             # Get the URL of the cover image.
             src = self.get_attr_value(attrs, 'src')
@@ -469,8 +550,19 @@ class HTMLMovieParser(ParserBase):
             self._is_languages = 0
         elif self._isplotoutline:
             self._plotoutline += data
-        elif self._inbch and sldata.startswith('plot outline:'):
-            self._isplotoutline = 1
+        elif self._in_series_title:
+            self._series_title += data
+        elif self._in_series_info:
+            self._series_info += data
+        elif self._inbch:
+            if sldata.startswith('plot outline:'):
+                self._isplotoutline = 1
+            elif sldata.startswith('tv series:'):
+                self._in_series_title = 1
+            elif sldata.startswith('original air date'):
+                self._in_series_info = 1
+        elif self._in_episode_title:
+            self._episode_title += data
         elif sldata.startswith('also known as'):
             self._is_akas = 1
         elif sldata.startswith('runtime:'):
@@ -497,7 +589,7 @@ class HTMLPlotParser(ParserBase):
     the akas.imdb.com server.  The final result will be a
     dictionary, with a 'plot' key, containing a list
     of string with the structure: 'summary_author <author@email>::summary'.
-    
+
     Example:
         pparser = HTMLPlotParser()
         result = pparser.parse(plot_summary_html_string)
@@ -577,7 +669,7 @@ class HTMLAwardsParser(ParserBase):
         self._aw_data = []
         # We're managing awards for a person or a movie?
         self.subject = 'title'
-    
+
     def _reset(self):
         """Reset the parser."""
         self._aw_data = []
@@ -681,7 +773,7 @@ class HTMLAwardsParser(ParserBase):
         self._to = []
         self._for = []
         self._cur_role = u''
-        
+
     def start_th(self, attrs):
         self._begin_aw = 0
 
@@ -855,7 +947,7 @@ class HTMLTaglinesParser(ParserBase):
             self._in_tl = 1
 
     def start_p(self, attrs): pass
-    
+
     def end_p(self):
         if self._in_tl and self._ctl:
             self._tl.append(self._ctl.strip())
@@ -958,7 +1050,7 @@ class HTMLAlternateVersionsParser(ParserBase):
 
     def end_ul(self):
         self._in_av = 0
-        
+
     def start_li(self, attrs):
         if self._in_av:
             self._in_avd = 1
@@ -1047,7 +1139,7 @@ class HTMLCrazyCreditsParser(ParserBase):
     def start_pre(self, attrs):
         if self._in_cc:
             self._in_cc2 = 1
-    
+
     def end_pre(self):
         if self._in_cc2:
             self.app()
@@ -1065,7 +1157,7 @@ class HTMLCrazyCreditsParser(ParserBase):
             self._cc.append(self._ccc.replace('\n', ' '))
             self._ccc = ''
             self._nrbr = 0
-    
+
     def _handle_data(self, data):
         if self._in_cc2:
             self._ccc += data
@@ -1101,14 +1193,14 @@ class HTMLGoofsParser(ParserBase):
 
     def end_ul(self):
         self._in_go = 0
-        
+
     def start_b(self, attrs):
         if self._in_go2:
             self._in_gok = 1
 
     def end_b(self):
         self._in_gok = 0
-            
+
     def start_li(self, attrs):
         if self._in_go:
             self._in_go2 = 1
@@ -1166,7 +1258,7 @@ class HTMLQuotesParser(ParserBase):
         name = self.get_attr_value(attrs, 'name')
         if name and name.startswith('qt'):
             self._in_quo2 = 1
-    
+
     def end_a(self): pass
 
     def do_hr(self, attrs):
@@ -1185,7 +1277,7 @@ class HTMLQuotesParser(ParserBase):
     def do_br(self, attrs):
         if self._in_quo and self._in_quo2 and self._cquo:
             self._cquo = '%s::' % self._cquo.strip()
-    
+
     def _handle_data(self, data):
         if self._in_quo and self._in_quo2:
             data = data.replace('\n', ' ')
@@ -1226,7 +1318,7 @@ class HTMLReleaseinfoParser(ParserBase):
             self._in_rl = 1
 
     def end_th(self): pass
-        
+
     def start_a(self, attrs):
         if self._in_rl:
             href = self.get_attr_value(attrs, 'href')
@@ -1239,7 +1331,7 @@ class HTMLReleaseinfoParser(ParserBase):
             if self._crl:
                 self._crl += '::'
             self._is_country = 0
-     
+
     def start_tr(self, attrs): pass
 
     def end_tr(self):
@@ -1353,7 +1445,7 @@ class HTMLRatingsParser(ParserBase):
 
     def end_p(self):
         self._in_p = 0
-    
+
     def start_a(self, attrs):
         href = self.get_attr_value(attrs, 'href')
         if href and href.startswith('ratings-'):
@@ -1362,7 +1454,7 @@ class HTMLRatingsParser(ParserBase):
 
     def end_a(self):
         self._in_demo_t = 0
-    
+
     def _handle_data(self, data):
         if self._in_b and data == 'Rating':
             self._in_total = 1
@@ -1380,7 +1472,7 @@ class HTMLRatingsParser(ParserBase):
                     sd = sd[:i]
                     try: sd = int(sd)
                     except (ValueError, OverflowError): pass
-                    if type(sd) is type(1):
+                    if type(sd) is type(0):
                         self._rank['top 250 rank'] = sd
             elif sdata.startswith('Arithmetic mean = '):
                 if sdata[-1] == '.': sdata = sdata[:-1]
@@ -1393,7 +1485,7 @@ class HTMLRatingsParser(ParserBase):
                 med = sdata[9:]
                 try: med = int(med)
                 except (ValueError, OverflowError): pass
-                if type(med) is type(1):
+                if type(med) is type(0):
                     self._rank['median'] = med
         if self._in_demo:
             if self._next_is_demo_vote:
@@ -1428,7 +1520,7 @@ class HTMLOfficialsitesParser(ParserBase):
 
     def _init(self):
         self.kind = 'official sites'
-    
+
     def _reset(self):
         """Reset the parser."""
         self._in_os = 0
@@ -1453,7 +1545,7 @@ class HTMLOfficialsitesParser(ParserBase):
     def start_ol(self, attrs):
         if self._in_os:
             self._in_os2 = 1
-    
+
     def end_ol(self):
         if self._in_os2:
             self._in_os2 = 0
@@ -1478,9 +1570,9 @@ class HTMLOfficialsitesParser(ParserBase):
                     if href.startswith('/'): href = href[1:]
                     href = 'http://akas.imdb.com/%s' % href
                 self._cosl = href
-        
+
     def end_a(self): pass
-    
+
     def _handle_data(self, data):
         if self._in_os3:
             self._cos += data
@@ -1534,7 +1626,7 @@ class HTMLConnectionParser(ParserBase):
         tn = self.re_imdbID.findall(href)
         if tn:
             self._cur_id = tn[-1]
-    
+
     def end_a(self): pass
 
     def do_br(self, attrs):
@@ -1571,7 +1663,7 @@ class HTMLTechParser(ParserBase):
 
     def _init(self):
         self.kind = 'something else'
-    
+
     def _reset(self):
         """Reset the parser."""
         self._tc = {}
@@ -1580,7 +1672,7 @@ class HTMLTechParser(ParserBase):
         self._indd = 0
         self._cur_sect = ''
         self._curdata = ['']
-    
+
     def get_data(self):
         """Return the dictionary."""
         if self.kind == 'locations':
@@ -1809,7 +1901,7 @@ class HTMLDvdParser(ParserBase):
             self._coldata += '::'
 
     def end_li(self): pass
-            
+
     def start_h3(self, attrs):
         if self._indvd:
             self._intitle = 1
@@ -1915,7 +2007,7 @@ class HTMLRecParser(ParserBase):
         self._intable = 0
         self._inb = 0
         self._cur_id = ''
-    
+
     def get_data(self):
         if not self._rec: return {}
         return {'recommendations': self._rec}
@@ -2093,7 +2185,7 @@ class HTMLAmazonReviewsParser(ParserBase):
         self._cur_text = ''
         self._cur_link = ''
         self._cur_revkind = ''
-    
+
     def get_data(self):
         """Return the dictionary."""
         if not self._reviews: return {}
@@ -2318,11 +2410,12 @@ class HTMLSalesParser(ParserBase):
         self._get_descr = 0
         self._in_a = 0
         self._in_dd = 0
+        self._cur_link_text = ''
 
     def get_data(self):
         if not self._sales: return {}
         return {'merchandising links': self._sales}
-    
+
     def start_h3(self, attrs):
         self._in_h3 = 1
         self._cur_type = ''
@@ -2337,7 +2430,7 @@ class HTMLSalesParser(ParserBase):
             clsl = cls.lower()
             if clsl == 'w_rowtable_colcover':
                 self._get_img = 1
-            elif clsl == 'w_rowtable_coldetails':
+            elif clsl in ('w_rowtable_colshop', 'w_rowtable_coldetails'):
                 self._get_descr = 1
                 self._cur_descr = ''
 
@@ -2349,6 +2442,10 @@ class HTMLSalesParser(ParserBase):
             self._get_img = 0
             src = self.get_attr_value(attrs, 'src')
             if src: self._cur_info['cover'] = src
+        if self._get_descr:
+            alttxt = self.get_attr_value(attrs, 'alt')
+            if alttxt:
+                self._cur_link_text = alttxt
 
     def end_img(self): pass
 
@@ -2358,11 +2455,15 @@ class HTMLSalesParser(ParserBase):
         self._cur_descr = self._cur_descr.strip()
         if self._cur_descr:
             self._cur_info['description'] = self._cur_descr
-            self._cur_descr = ''
-            if self._cur_info:
-                self._sales.setdefault(self._cur_type,
-                                        []).append(self._cur_info)
-                self._cur_info = {}
+        self._cur_descr = ''
+        self._cur_link_text = self._cur_link_text.strip()
+        if self._cur_link_text:
+            self._cur_info['link-text'] = self._cur_link_text
+        self._cur_link_text = ''
+        if self._cur_info:
+            self._sales.setdefault(self._cur_type,
+                                    []).append(self._cur_info)
+            self._cur_info = {}
 
     def start_dt(self, attrs):
         self._in_dt = 1
@@ -2402,6 +2503,169 @@ class HTMLSalesParser(ParserBase):
             self._cur_descr += data
 
 
+class HTMLEpisodesParser(ParserBase):
+    """Parser for the "episode list" page of a given movie.
+    The page should be provided as a string, as taken from
+    the akas.imdb.com server.  The final result will be a
+    dictionary, with a key for every relevant section.
+
+    Example:
+        eparser = HTMLEpisodesParser()
+        result = eparser.parse(episodes_html_string)
+    """
+
+    # Do not gather names and titles references.
+    getRefs = 0
+
+    def _reset(self):
+        self._in_html_title = 0
+        self._series = None
+        self._html_title = ''
+        self._episodes = {}
+        self._in_h1 = 0
+        self._in_episodes_h1 = 0
+        self._cur_season = 0
+        self._cur_episode = None
+        self._in_episodes = 0
+        self._in_td_eps = 0
+        self._in_td_title = 1
+        self._in_title = 0
+        self._cur_title = ''
+        self._curid = ''
+        self._eps_counter = 1
+        self._cur_info = 'title'
+        self._info = {}
+        self._info_text = ''
+
+    def get_data(self):
+        if self._episodes: return {'episodes': self._episodes}
+        else: return {}
+
+    def start_title(self, attrs):
+        self._in_html_title = 1
+
+    def end_title(self):
+        self._in_html_title = 0
+        title = self._html_title
+        if title.lower().startswith('episodes for'):
+            title = title[12:].strip()
+        if title:
+            self._series = Movie(data=analyze_title(title, canonical=1),
+                                accessSystem='http')
+        self._html_title = ''
+        self._series
+
+    def start_h1(self, attrs):
+        self._in_h1 = 1
+
+    def end_h1(self):
+        self._in_h1 = 0
+
+    def start_table(self, attrs):
+        if self._in_episodes_h1:
+            self._in_episodes = 1
+
+    def end_table(self):
+        self._in_episodes = 0
+
+    def start_a(self, attrs):
+        if not self._in_episodes: return
+        if self._in_td_title:
+            href = self.get_attr_value(attrs, 'href')
+            if href and href.startswith('/title/tt'):
+                curid = self.re_imdbID.findall(href)
+                if curid:
+                    self._in_title = 1
+                    self._cur_title = ''
+                    self._curid = curid[-1]
+                    return
+        name = self.get_attr_value(attrs, 'name')
+        if name and name.lower().startswith('season-'):
+            cs = name[7:]
+            try: cs = int(cs)
+            except: pass
+            self._cur_season = cs
+            self._eps_counter = 1
+
+    def end_a(self):
+        if self._in_title: self._in_title = 0
+
+    def start_td(self, attrs):
+        if self._in_episodes:
+            self._in_td_eps = 1
+
+    def end_td(self):
+        if self._in_td_title:
+            self._in_td_eps = 0
+            self._in_td_title = 0
+            self._info_text = self._info_text.strip()
+            if self._info_text and self._cur_info != 'title':
+                self._info[self._cur_info] = self._info_text
+                self._info_text = ''
+            if self._cur_title and self._curid:
+                m = Movie(title=self._cur_title,
+                            movieID=str(self._curid),
+                            accessSystem='http')
+                m['kind'] = 'episode'
+                if self._cur_season not in self._episodes:
+                    self._episodes[self._cur_season] = {}
+                ce = self._cur_episode
+                if ce is None:
+                    ce = self._eps_counter
+                if self._series is not None:
+                    m['episode of'] = self._series
+                m['season'] = self._cur_season
+                m['episode'] = ce
+                self._episodes[self._cur_season][ce] = m
+                self._eps_counter += 1
+                self._cur_title = ''
+                self._curid = ''
+                self._cur_episode = None
+                for key, value in self._info.items():
+                    if key == 'original air date':
+                        if value[-4:].isdigit() and \
+                                    m.get('year', '????') == '????':
+                            m['year'] = value[-4:]
+                    m[key] = value
+            self._cur_info = 'title'
+            self._info_text = ''
+            self._info = {}
+
+    def do_br(self, attrs):
+        if self._in_td_title:
+            self._info_text = self._info_text.strip()
+            if self._info_text and self._cur_info != 'title':
+                self._info[self._cur_info] = self._info_text
+                self._info_text = ''
+            if self._cur_info == 'title':
+                self._cur_info = 'original air date'
+            elif self._cur_info == 'original air date':
+                self._cur_info = 'plot'
+
+    def _handle_data(self, data):
+        if self._in_h1 and data.strip().lower().startswith('episodes'):
+            self._in_episodes_h1 = 1
+        elif self._in_td_eps:
+            ldata = data.lower()
+            if ldata.find('season') != -1:
+                fe = ldata.find('episode')
+                if fe != -1:
+                    self._in_td_title = 1
+                    ce = ''
+                    for char in ldata[fe+8:].strip():
+                        if char.isdigit(): ce += char
+                        else: break
+                    try: ce = int(ce)
+                    except: ce = None
+                    self._cur_episode = ce
+        elif self._in_html_title:
+            self._html_title += data
+        if self._in_title:
+            self._cur_title += data
+        elif self._cur_info != 'title' and self._in_td_title:
+            self._info_text += data
+
+
 # The used instances.
 movie_parser = HTMLMovieParser()
 plot_parser = HTMLPlotParser()
@@ -2444,4 +2708,5 @@ news_parser = HTMLNewsParser()
 amazonrev_parser = HTMLAmazonReviewsParser()
 guests_parser = HTMLGuestsParser()
 sales_parser = HTMLSalesParser()
+episodes_parser = HTMLEpisodesParser()
 
