@@ -6,7 +6,7 @@ IMDb's data through a local installation.
 the imdb.IMDb function will return an instance of this class when
 called with the 'accessSystem' argument set to "local" or "files".
 
-Copyright 2004-2006 Davide Alberani <da@erlug.linux.it> 
+Copyright 2004-2006 Davide Alberani <da@erlug.linux.it>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,29 +33,66 @@ from imdb.utils import analyze_title, analyze_name, re_episodes, \
                         sortMovies, sortPeople
 from imdb.Person import Person
 from imdb.Movie import Movie
-
 from personParser import getFilmography, getBio, getAkaNames
 from movieParser import getLabel, getMovieCast, getAkaTitles, parseMinusList, \
                         getPlot, getRatingData, getMovieMisc, getTaglines, \
                         getQuotes, getMovieLinks, getBusiness, getLiterature, \
                         getLaserdisc
-
 from utils import getFullIndex, KeyFScan, latin2utf
-
 from imdb.parser.common.locsql import IMDbLocalAndSqlAccessSystem, \
                                         titleVariations, nameVariations
 
-_ltype = type([])
-_dtype = type({})
-_stypes = (type(u''), type(''))
+try:
+    from imdb.parser.common.cutils import get_episodes
+except ImportError:
+    import warnings
+    warnings.warn('Unable to import the cutils.get_episodes function.'
+                    '  Retrieving episodes list of tv series will be'
+                    ' a bit slower.')
+
+    def get_episodes(movieID, indexFile, keyFile):
+        if movieID < 0:
+            raise IMDbDataAccessError, "movieID must be positive."
+        try:
+            ifile = open(indexFile, 'rb')
+        except IOError, e:
+            raise IMDbDataAccessError, str(e)
+        ifile.seek(4L*movieID, 0)
+        indexStr = ifile.read(4)
+        ifile.close()
+        if len(indexStr) != 4:
+            raise IMDbDataAccessError, \
+                    "unable to read indexFile; movieID too high?"
+        kfIndex = 0L
+        for i in (0, 1, 2, 3):
+            kfIndex |= ord(indexStr[i]) << i*8L;
+        try:
+            kfile = open(keyFile, 'rt')
+        except IOError, e:
+            raise IMDbDataAccessError, str(e)
+        kfile.seek(kfIndex, 0)
+        seriesTitle = kfile.readline().split('|')[0].strip()
+        if seriesTitle[0:1] != '"' or seriesTitle[-1:] != ')':
+            return []
+        stLen = len(seriesTitle)
+        results = []
+        for line in kfile:
+            if not line.strip(): break
+            epsTitle, epsID = line.split('|')
+            if epsTitle[:stLen] != seriesTitle: break
+            if epsTitle[stLen+1] != '{' or epsTitle[-1] != '}':
+                break
+            results.append((int(epsID.strip(), 16), epsTitle))
+        kfile.close()
+        return results
 
 try:
-    from imdb.parser.common.ratober import search_name
+    from imdb.parser.common.cutils import search_name
 
     def _scan_names(keyFile, name1, name2, name3, results=0):
-        """Scan the given file, using the ratober.search_name
+        """Scan the given file, using the cutils.search_name
         C function, for name variations."""
-        # the search_name function in the ratober C module manages
+        # the search_name function in the cutils C module manages
         # latin_1 encoded strings.
         name1, name2, name3 = [x.encode('latin_1', 'replace')
                                 for x in name1, name2, name3]
@@ -63,11 +100,11 @@ try:
         res = []
         for x in sn:
             tmpd = analyze_name(latin2utf(x[2]))
-            res.append((x[0], (x[1], tmpd['name'], tmpd.get('imdbIndex'))))
+            res.append((x[0], (x[1], tmpd)))
         return res
 except ImportError:
     import warnings
-    warnings.warn('Unable to import the ratober.search_name function.'
+    warnings.warn('Unable to import the cutils.search_name function.'
                     '  Searching names using the "local" data access system'
                     ' will be REALLY slow.')
 
@@ -82,7 +119,7 @@ except ImportError:
             ls = line.split('|')
             if not ls[0]: continue
             named = analyze_name(latin2utf(ls[0]))
-            yield (long(ls[1], 16), named['name'], named.get('imdbIndex'))
+            yield (long(ls[1], 16), named)
         kf.close()
 
     def _scan_names(keyFile, name1, name2, name3, results=0):
@@ -92,10 +129,10 @@ except ImportError:
                             name1, name2, name3, results)
 
 try:
-    from imdb.parser.common.ratober import search_title
+    from imdb.parser.common.cutils import search_title
 
     def _scan_titles(keyFile, title1, title2, title3, results=0):
-        """Scan the given file, using the ratober.search_title
+        """Scan the given file, using the cutils.search_title
         C function, for title variations."""
         title1, title2, title3 = [x.encode('latin_1', 'replace')
                                     for x in title1, title2, title3]
@@ -103,34 +140,46 @@ try:
         res = []
         for x in st:
             tmpd = analyze_title(latin2utf(x[2]))
-            res.append((x[0], (x[1], tmpd['title'],
-                        tmpd.get('imdbIndex'), tmpd['kind'], tmpd.get('year'))))
+            res.append((x[0], (x[1], tmpd)))
         return res
 except ImportError:
     import warnings
-    warnings.warn('Unable to import the ratober.search_title function.'
+    warnings.warn('Unable to import the cutils.search_title function.'
                     '  Searching titles using the "local" data access system'
                     ' will be REALLY slow.')
 
     from imdb.parser.common.locsql import scan_titles
 
-    def _readTitlesKeyFile(keyFile):
+    def _readTitlesKeyFile(keyFile, searchingEpisode=0, lenTitle=1):
         """Iterate over the given file, returning tuples suited for
         the common.locsql.scan_titles function."""
+        STRING_MAXLENDIFFER = 0.7
+        lenTitle = float(lenTitle)
         try: kf = open(keyFile, 'r')
         except IOError, e: raise IMDbDataAccessError, str(e)
         for line in kf:
             ls = line.split('|')
-            if not ls[0]: continue
-            titled = analyze_title(latin2utf(ls[0]))
-            yield (long(ls[1], 16), titled['title'],
-                    titled.get('imdbIndex'), titled['kind'], titled.get('year'))
+            t = ls[0]
+            if not t: continue
+            l_t = len(t)
+            if (lenTitle > l_t): threshold = l_t / lenTitle
+            else: threshold = lenTitle / l_t
+            # don't compare too different lenght strings.
+            if threshold < STRING_MAXLENDIFFER: continue
+            if searchingEpisode:
+                if t[-1] != '}': continue
+            elif t[-1] == '}': continue
+            titled = analyze_title(latin2utf(t))
+            yield (long(ls[1], 16), titled)
         kf.close()
 
     def _scan_titles(keyFile, title1, title2, title3, results=0):
         """Scan the given file, using the common.locsql.scan_titles
         pure-Python function, for title variations."""
-        return scan_titles(_readTitlesKeyFile(keyFile),
+        se = 0
+        if title3 and title3[-1] == '}': se = 1
+        return scan_titles(_readTitlesKeyFile(keyFile, searchingEpisode=se,
+                            lenTitle=len(title3 or title1)),
                             title1, title2, title3, results)
 
 
@@ -172,7 +221,7 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
     def get_lastMovieID(self):
         """Return the last movieID"""
         return self._get_lastID('%stitles.index' % self.__db)
-    
+
     def get_lastPersonID(self):
         """Return the last personID"""
         return self._get_lastID('%snames.index' % self.__db)
@@ -214,7 +263,8 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         """
         titline = getLabel(movieID, '%stitles.index' % self.__db,
                             '%stitles.key' % self.__db)
-        return self._httpMovieID(titline)
+        if titline is None: return None
+        return self.title2imdbID(titline)
 
     def get_imdbPersonID(self, personID):
         """Translate a personID in an imdbID.
@@ -223,7 +273,8 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         """
         name = getLabel(personID, '%snames.index' % self.__db,
                         '%snames.key' % self.__db)
-        return self._httpPersonID(name)
+        if name is None: return None
+        return self.name2imdbID(name)
 
     def do_adult_search(self, doAdult):
         """If set to 0 or False, movies in the Adult category are not
@@ -253,14 +304,7 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                 if 'Adult' not in genres: newlist.append(entry)
             res[:] = newlist
             if results > 0: res[:] = res[:results]
-        # Purge empty imdbIndex and year.
-        returnl = []
-        for x in res:
-            tmpd = {'title': x[1], 'kind': x[3]}
-            if x[2]: tmpd['imdbIndex'] = x[2]
-            if x[4]: tmpd['year'] = x[4]
-            returnl.append((x[0], tmpd))
-        return returnl
+        return res
 
     def get_movie_main(self, movieID):
         # Information sets provided by this method.
@@ -339,7 +383,7 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
             episodes = re_episodes.findall(rt)
             if episodes:
                 res['runtimes'][0] = re_episodes.sub('', rt)
-                res['episodes'] = episodes[0]
+                res['number of episodes'] = episodes[0]
         # AKA titles.
         akas = getAkaTitles(movieID,
                     '%saka-titles.data' % self.__db,
@@ -357,6 +401,20 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                 nt = self._changeAKAencoding(n, t)
                 if nt is not None: akas[i] = '%s::%s' % (nt, n)
             res['akas'] = akas
+        if res.get('kind') == 'episode':
+            # Things to do if this is a tv series episode.
+            episodeOf = res.get('episode of')
+            if episodeOf is not None:
+                parentSeries = Movie(data=res['episode of'],
+                                            accessSystem='local')
+                seriesID = self._getTitleID(parentSeries.get(
+                                            'long imdb canonical title'))
+                parentSeries.movieID = seriesID
+                res['episode of'] = parentSeries
+            if not res.get('year'):
+                year = getFullIndex('%smovies.data' % self.__db,
+                                    movieID, kind='moviedata', rindex=1)
+                if year: res['year'] = year
         return {'data': res, 'info sets': infosets}
 
     def get_movie_plot(self, movieID):
@@ -406,7 +464,7 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         if goo: return {'data': {'goofs': goo},
                         'titlesRefs': trefs, 'namesRefs': nrefs}
         return {'data': {}}
-    
+
     def get_movie_soundtrack(self, movieID):
         goo = parseMinusList(movieID, '%ssoundtracks.data' % self.__db,
                             '%ssoundtracks.index' % self.__db)
@@ -469,13 +527,13 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         if mb: return {'data': {'business': mb},
                         'titlesRefs': trefs, 'namesRefs': nrefs}
         return {'data': {}}
-    
+
     def get_movie_literature(self, movieID):
         ml = getLiterature(movieID, '%sliterature.index' % self.__db,
                             '%sliterature.data' % self.__db)
         if ml: return {'data': {'literature': ml}}
         return {'data': {}}
-    
+
     def get_movie_laserdisc(self, movieID):
         ml = getLaserdisc(movieID, '%slaserdisc.index' % self.__db,
                             '%slaserdisc.data' % self.__db)
@@ -484,6 +542,39 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                         'titlesRefs': trefs, 'namesRefs': nrefs}
         return {'data': {}}
 
+    def _buildEpisodes(self, eps_list, parentID):
+        episodes = {}
+        parentTitle = getLabel(parentID, '%stitles.index' % self.__db,
+                            '%stitles.key' % self.__db)
+        parentSeries = Movie(title=parentTitle,
+                            movieID=parentID, accessSystem='local')
+        for episodeID, episodeTitle in eps_list:
+            episodeTitle = unicode(episodeTitle, 'latin_1', 'replace')
+            data = analyze_title(episodeTitle, canonical=1)
+            m = Movie(data=data, movieID=episodeID, accessSystem='local')
+            m['episode of'] = parentSeries
+            if data.get('year') is None:
+                year = getFullIndex('%smovies.data' % self.__db,
+                                    key=episodeID, kind='moviedata', rindex=1)
+                if year: m['year'] = year
+            season = data.get('season', 'UNKNOWN')
+            if not episodes.has_key(season): episodes[season] = {}
+            ep_number = data.get('episode')
+            if ep_number is None:
+                ep_number = max((episodes[season].keys() or [0])) + 1
+            episodes[season][ep_number] = m
+        return episodes
+
+    def get_movie_episodes(self, movieID):
+        try:
+            me = get_episodes(movieID, '%stitles.index' % self.__db,
+                                        '%stitles.key' % self.__db)
+        except IOError, e:
+            raise IMDbDataAccessError, str(e)
+        if me:
+            return {'data': {'episodes': self._buildEpisodes(me, movieID)}}
+        return {'data': {}}
+
     def _search_person(self, name, results):
         name = name.strip()
         if not name: return []
@@ -494,14 +585,7 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                             name1, name2, name3, resultsST)
         if results > 0: res[:] = res[:results]
         res[:] = [x[1] for x in res]
-        # Purge empty imdbIndex and year.
-        returnl = []
-        for x in res:
-            tmpd = {'name': x[1]}
-            if x[2]: tmpd['imdbIndex'] = x[2]
-            returnl.append((x[0], tmpd))
-        return returnl
-
+        return res
 
     def get_person_main(self, personID):
         infosets = ('main', 'biography', 'other works')
@@ -522,6 +606,8 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
         #      retrieve the movieID!
         #      A cleaner solution, would be to NOT return Movies object
         #      at first, from the getBio() function.
+        # XXX: anyway, this is no more needed, since "guest appearances"
+        #      are gone with the new tv series episodes support.
         if res.has_key('notable tv guest appearances'):
             nl = []
             for m in res['notable tv guest appearances']:
@@ -538,7 +624,9 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                 'titlesRefs': trefs, 'namesRefs': nrefs}
 
     def get_person_filmography(self, personID):
+        infosets = ('filmography', 'episodes')
         res = {}
+        episodes = {}
         works = ('actor', 'actresse', 'producer', 'writer',
                 'cinematographer', 'composer', 'costume-designer',
                 'director', 'editor', 'miscellaneou', 'production-designer')
@@ -565,14 +653,39 @@ class IMDbLocalAccessSystem(IMDbLocalAndSqlAccessSystem):
                     params['doWriters'] = 1
                 params['dataF'] = '%s%ss.data' % (self.__db, key)
                 data = getFilmography(**params)
-                data.sort(sortMovies)
-                res[name] = data
-        return {'data': res}
-
-    def get_person_biography(self, personID):
-        return self.get_person_main(personID)
-
-    def get_person_other_works(self, personID):
-        return self.get_person_main(personID)
+                movies = []
+                eps = []
+                # Split normal titles from episodes.
+                for d in data:
+                    if d.get('kind') != 'episode':
+                        movies.append(d)
+                    else:
+                        eps.append(d)
+                movies.sort(sortMovies)
+                if movies:
+                    res[name] = movies
+                for e in eps:
+                    series = Movie(data=e['episode of'], accessSystem='local')
+                    seriesID = self._getTitleID(series.get(
+                                                'long imdb canonical title'))
+                    series.movieID = seriesID
+                    if not e.get('year'):
+                        year = getFullIndex('%smovies.data' % self.__db,
+                                            e.movieID, kind='moviedata',
+                                            rindex=1)
+                        if year: e['year'] = year
+                    if not e.currentRole and name not in ('actor', 'actress'):
+                        if e.notes: e.notes = ' %s' % e.notes
+                        e.notes = '[%s]%s' % (name, e.notes)
+                    episodes.setdefault(series, []).append(e)
+        if episodes:
+            for k in episodes:
+                episodes[k].sort(sortMovies)
+                episodes[k].reverse()
+            res['episodes'] = episodes
+        return {'data': res, 'info sets': tuple(infosets)}
 
+    get_person_biography = get_person_main
+    get_person_other_works = get_person_main
+    get_person_episodes = get_person_filmography
 
