@@ -20,12 +20,15 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """
 
+from __future__ import generators
 import re
+import string
+from types import UnicodeType, StringType, ListType, TupleType, DictType
 from copy import copy, deepcopy
+from time import strptime, strftime
 
 from imdb._exceptions import IMDbParserError
 
-
 # The regular expression for the "long" year format of IMDb, like
 # "(1998)" and "(1986/II)", where the optional roman number (that I call
 # "imdbIndex" after the slash is used for movies with the same title
@@ -39,6 +42,8 @@ re_index = re.compile(r'^\(([IVXLCDM]+)\)$')
 # Match the number of episodes.
 re_episodes = re.compile('\s?\((\d+) episodes\)', re.I)
 
+re_episode_info = re.compile(r'{(.+?)?\s?(\([0-9\?]{4}-[0-9\?]{1,2}-[0-9\?]{1,2}\))?\s?(\(#[0-9]+\.[0-9]+\))?}')
+
 # Common suffixes in surnames.
 _sname_suffixes = ('de', 'la', 'der', 'den', 'del', 'y', 'da', 'van',
                     'e', 'von', 'the', 'di', 'du', 'el', 'al')
@@ -46,14 +51,14 @@ _sname_suffixes = ('de', 'la', 'der', 'den', 'del', 'y', 'da', 'van',
 def canonicalName(name):
     """Return the given name in canonical "Surname, Name" format.
     It assumes that name is in the 'Name Surname' format."""
-    # XXX: some statistics (over 1698193 names):
-    #      - just a surname:                 36614
-    #      - single surname, single name:  1461621
-    #      - composed surname, composed name: 6126
-    #      - composed surname, single name:  44597
-    #        (2: 39998, 3: 4175, 4: 356)
-    #      - single surname, composed name: 149235
-    #        (2: 143522, 3: 4787, 4: 681, 5: 165)
+    # XXX: some statistics (over 1852406 names):
+    #      - just a surname:                 51921
+    #      - single surname, single name:  1792759
+    #      - composed surname, composed name: 7726
+    #      - composed surname, single name:  55623
+    #        (2: 49259, 3: 5502, 4: 551)
+    #      - single surname, composed name: 186604
+    #        (2: 178315, 3: 6573, 4: 1219, 5: 352)
     # Don't convert names already in the canonical format.
     if name.find(', ') != -1: return name
     sname = name.split(' ')
@@ -100,7 +105,7 @@ def analyze_name(name, canonical=0):
     keys, from the given string.
     If canonical is true, it tries to convert the  name to
     the canonical "Surname, Name" format.
-    
+
     raise an IMDbParserError exception if the name is not valid.
     """
     original_n = name
@@ -129,7 +134,8 @@ def build_name(name_dict, canonical=0):
     If canonical is not set, the name is returned in the normal
     "Name Surname" format.
     """
-    name = name_dict.get('name', '')
+    name = name_dict.get('canonical name') or name_dict.get('name', '')
+    if not name: return u''
     if not canonical:
         name = normalizeName(name)
     imdbIndex = name_dict.get('imdbIndex')
@@ -139,16 +145,35 @@ def build_name(name_dict, canonical=0):
 
 
 # List of articles.
-# XXX: are 'agapi mou' and  'liebling' articles?
-# XXX: removed 'en', 'to', 'as', 'et', 'des', 'al', 'egy', 'ye', 'da'
-#      and "'n" because they are more commonly used as non-articles
-#      at the begin of a title.
-_articles = ('the', 'la', 'a', 'die', 'der', 'le', 'el', "l'", 'il',
-            'das', 'les', 'i', 'o', 'ein', 'un', 'los', 'de', 'an',
-            'una', 'eine', 'las', 'den', 'gli', 'het', 'lo',
-            'os', 'az', 'ha-', 'een', 'det', 'oi', 'ang', 'ta',
-            'al-', 'dem', 'uno', "un'", 'ett', 'mga', u'\xcf', u'\xc7',
-            'eines', 'els', u'\xd4\xef', u'\xcf\xe9')
+# XXX: Managing titles in a lot of different languages, a function to recognize
+# an initial article can't be perfect; sometimes we'll stumble upon a short
+# word that is an article in some language, but it's not in another; in these
+# situations we have to choose if we want to interpret this little word
+# as an article or not (remember that we don't know what the original language
+# of the title was).
+# Example: 'da' is an article in (I think) Dutch and it's used as an article
+# even in some American slangs.  Unfortunately it's also a preposition in
+# Italian, and it's widely used in Mandarin (for whatever it means!).
+# Running a script over the whole list of titles (and aliases), I've found
+# that 'da' is used as an article only 20 times, and as another thing 255
+# times, so I've decided to _always_ consider 'da' as a non article.
+#
+# Here is a list of words that are _never_ considered as articles, complete
+# with the cound of times they are used in a way or another:
+# 'en' (314 vs 507), 'to' (236 vs 589), 'as' (183 vs 231), 'et' (67 vs 79),
+# 'des' (69 vs 123), 'al' (57 vs 247), 'egy' (28 vs 32), 'ye' (14 vs 55),
+# 'da' (20 vs 255), "'n" (7 vs 12)
+#
+# I've left in the list 'i' (1614 vs 1707) and 'uno' (49 vs 51)
+# I'm not sure what '-al' is, and so I've left it out...
+#
+# List of articles:
+_articles = ('the', 'la', 'a', 'die', 'der', 'le', 'el',
+            "l'", 'il', 'das', 'les', 'o', 'ein', 'i', 'un', 'los', 'de',
+            'an', 'una', 'las', 'eine', 'den', 'gli', 'het', 'os', 'lo',
+            'az', 'det', 'ha-', 'een', 'ang', 'oi', 'ta', 'al-', 'dem',
+            'mga', 'uno', "un'", 'ett', u'\xcf', 'eines', u'\xc7', 'els',
+            u'\xd4\xef', u'\xcf\xe9')
 
 # Articles in a dictionary.
 _articlesDict = dict([(x, x) for x in _articles])
@@ -192,26 +217,124 @@ def normalizeTitle(title):
     return title
 
 
-def analyze_title(title, canonical=0):
+def _split_series_episode(title):
+    """Return the series and the episode titles; if this is not a
+    series' episode, the returned series title is empty.
+    This function recognize two different styles:
+        "The Series" An Episode (2005)
+        "The Series" (2004) {An Episode (2005) (#season.episode)}"""
+    series_title = ''
+    episode_or_year = ''
+    if title[-1:] == '}':
+        # Title of the episode, as in the plain text data files.
+        begin_eps = title.rfind('{')
+        if begin_eps == -1: return '', ''
+        series_title = title[:begin_eps].rstrip()
+        # episode_or_year is returned with the {...}
+        episode_or_year = title[begin_eps:]
+        if episode_or_year[:12] == '{SUSPENDED}}': return '', ''
+    # XXX: works only with tv series; it's still unclear whether
+    #      IMDb will support episodes for tv mini series and tv movies...
+    elif title[0:1] == '"':
+        second_quot = title[1:].find('"') + 2
+        if second_quot != 1: # a second " was found.
+            episode_or_year = title[second_quot:].lstrip()
+            first_char = episode_or_year[0:1]
+            if not first_char: return '', ''
+            if first_char != '(':
+                # There is not a (year) but the title of the episode;
+                # that means this is an episode title, as returned by
+                # the web server.
+                series_title = title[:second_quot]
+            ##elif episode_or_year[-1:] == '}':
+            ##        # Title of the episode, as in the plain text data files.
+            ##        begin_eps = episode_or_year.find('{')
+            ##        if begin_eps == -1: return series_title, episode_or_year
+            ##        series_title = title[:second_quot+begin_eps].rstrip()
+            ##        # episode_or_year is returned with the {...}
+            ##        episode_or_year = episode_or_year[begin_eps:]
+    return series_title, episode_or_year
+
+
+def is_series_episode(title):
+    """Return True if 'title' is an series episode."""
+    title = title.strip()
+    if _split_series_episode(title)[0]: return 1
+    return 0
+
+
+def analyze_title(title, canonical=None,
+                    canonicalSeries=0, canonicalEpisode=0):
     """Analyze the given title and return a dictionary with the
     "stripped" title, the kind of the show ("movie", "tv series", etc.),
     the year of production and the optional imdbIndex (a roman number
     used to distinguish between movies with the same title and year).
     If canonical is true, the title is converted to the canonical
     format.
-    
+
     raise an IMDbParserError exception if the title is not valid.
     """
+    if canonical is not None:
+        canonicalSeries = canonicalEpisode = canonical
     original_t = title
+    result = {}
     title = title.strip()
     year = ''
     kind = ''
     imdbIndex = ''
+    series_title, episode_or_year = _split_series_episode(title)
+    if series_title:
+        # It's an episode of a series.
+        series_d = analyze_title(series_title, canonical=canonicalEpisode)
+        oad = sen = ep_year = ''
+        # Plain text data files format.
+        if episode_or_year[0:1] == '{' and episode_or_year[-1:] == '}':
+            match = re_episode_info.findall(episode_or_year)
+            if match:
+                # Episode title, original air date and #season.episode
+                episode_or_year, oad, sen = match[0]
+                if not oad:
+                    # No year, but the title is something like (2005-04-12)
+                    if episode_or_year and episode_or_year[0] == '(' and \
+                                    episode_or_year[-1:] == ')' and \
+                                    episode_or_year[1:2] != '#':
+                        oad = episode_or_year
+                        if oad[1:5] and oad[5:6] == '-':
+                            ep_year = oad[1:5]
+                if not oad and not sen and episode_or_year.startswith('(#'):
+                    sen = episode_or_year
+        elif episode_or_year.startswith('Episode dated'):
+            oad = episode_or_year[14:]
+            if oad[-4:].isdigit():
+                ep_year = oad[-4:]
+        episode_d = analyze_title(episode_or_year, canonical=canonicalEpisode)
+        episode_d['kind'] = 'episode'
+        episode_d['episode of'] = series_d
+        if oad:
+            episode_d['original air date'] = oad[1:-1]
+            if ep_year and episode_d.get('year') is None:
+                episode_d['year'] = ep_year
+        if sen:
+            seas, epn = sen[2:-1].split('.')
+            if seas:
+                # Set season and episode.
+                try: seas = int(seas)
+                except: pass
+                try: epn = int(epn)
+                except: pass
+                episode_d['season'] = seas
+                episode_d['episode'] = epn
+        return episode_d
     # First of all, search for the kind of show.
-    # XXX: Number of entries at 27 jan 2005:
-    #      {'(mini)': 4858, '(V)': 35501, '(VG)': 4301, '(TV)': 58443}
-    #      tv series: 42454
-    # XXX: more up-to-date statistics: http://us.imdb.com/database_statistics
+    # XXX: Number of entries at 18 Mar 2006:
+    #      movie:        344,892
+    #      episode:      272,862
+    #      tv movie:      53,269
+    #      tv series:     37,065
+    #      video movie:   44,062
+    #      tv mini series: 4,757
+    #      video game:     4,472
+    #      More up-to-date statistics: http://us.imdb.com/database_statistics
     if title.endswith('(TV)'):
         kind = 'tv movie'
         title = title[:-4].rstrip()
@@ -245,9 +368,10 @@ def analyze_title(title, canonical=0):
         raise IMDbParserError, 'invalid title: "%s"' % original_t
     if canonical:
         title = canonicalTitle(title)
-    # 'kind' is one in ('movie', 'tv series', 'tv mini series', 'tv movie'
-    #                   'video movie', 'video game')
-    result = {'title': title, 'kind': kind or 'movie'}
+    # 'kind' is one in ('movie', 'episode', 'tv series', 'tv mini series',
+    #                   'tv movie', 'video movie', 'video game')
+    result['title'] = title
+    result['kind'] = kind or 'movie'
     if year and year != '????':
         result['year'] = str(year)
     if imdbIndex:
@@ -255,25 +379,96 @@ def analyze_title(title, canonical=0):
     return result
 
 
-def build_title(title_dict, canonical=0):
+_web_format = '%d %B %Y'
+_ptdf_format = '(%Y-%m-%d)'
+def _convertTime(title, fromPTDFtoWEB=1):
+    """Convert a time expressed in the pain text data files, to
+    the 'Episode dated ...' format used on the web site; if
+    fromPTDFtoWEB is false, the inverted conversion is applied."""
+    try:
+        if fromPTDFtoWEB:
+            from_format = _ptdf_format
+            to_format = _web_format
+        else:
+            from_format = 'Episode dated %s' % _web_format
+            to_format = _ptdf_format
+        t = strptime(title, from_format)
+        title = strftime(to_format, t)
+        if fromPTDFtoWEB:
+            if title[0] == '0': title = title[1:]
+            title = 'Episode dated %s' % title
+    except ValueError:
+        pass
+    return title
+
+
+def build_title(title_dict, canonical=None,
+                canonicalSeries=0, canonicalEpisode=0, ptdf=0, _doYear=1):
     """Given a dictionary that represents a "long" IMDb title,
     return a string.
 
     If canonical is not true, the title is returned in the
     normal format.
+
+    If ptdf is true, the plain text data files format is used.
     """
-    title = title_dict.get('title', '')
+    if canonical is not None:
+        canonicalSeries = canonicalEpisode = canonical
+    pre_title = ''
+    kind = title_dict.get('kind')
+    episode_of = title_dict.get('episode of')
+    if kind == 'episode' and episode_of is not None:
+        # Works with both Movie instances and plain dictionaries.
+        doYear = 0
+        if ptdf:
+            doYear = 1
+        pre_title = build_title(episode_of, canonical=canonicalSeries,
+                                ptdf=0, _doYear=doYear)
+        ep_dict = {'title': title_dict.get('title', ''),
+                    'imdbIndex': title_dict.get('imdbIndex')}
+        ep_title = ep_dict['title']
+        if not ptdf:
+            doYear = 1
+            ep_dict['year'] = title_dict.get('year') or '????'
+            if ep_title[0:1] == '(' and ep_title[-1:] == ')' and \
+                    ep_title[1:5].isdigit():
+                ep_dict['title'] = _convertTime(ep_title, fromPTDFtoWEB=1)
+        else:
+            doYear = 0
+            if ep_title.startswith('Episode dated'):
+                ep_dict['title'] = _convertTime(ep_title, fromPTDFtoWEB=0)
+        episode_title = build_title(ep_dict,
+                            canonical=canonicalEpisode, ptdf=ptdf,
+                            _doYear=doYear)
+        if ptdf:
+            oad = title_dict.get('original air date', '')
+            if len(oad) == 10 and oad[4] == '-' and oad[7] == '-' and \
+                        episode_title.find(oad) == -1:
+                episode_title += ' (%s)' % oad
+            seas = title_dict.get('season')
+            if seas is not None:
+                episode_title += ' (#%s' % seas
+                episode = title_dict.get('episode')
+                if episode is not None:
+                    episode_title += '.%s' % episode
+                episode_title += ')'
+            episode_title = '{%s}' % episode_title
+        return '%s %s' % (pre_title, episode_title)
+    title = title_dict.get('canonical title') or title_dict.get('title', '')
+    if not title: return u''
     if not canonical:
         title = normalizeTitle(title)
-    kind = title_dict.get('kind')
-    imdbIndex = title_dict.get('imdbIndex')
-    year = title_dict.get('year', '????')
+    if pre_title:
+        title = '%s %s' % (pre_title, title)
     if kind in ('tv series', 'tv mini series'):
         title = '"%s"' % title
-    title += ' (%s' % year
-    if imdbIndex:
-        title += '/%s' % imdbIndex
-    title += ')'
+    if _doYear:
+        imdbIndex = title_dict.get('imdbIndex')
+        year = title_dict.get('year') or '????'
+        title += ' (%s' % year
+        if imdbIndex:
+            title += '/%s' % imdbIndex
+        title += ')'
     if kind:
         if kind == 'tv movie':
             title += ' (TV)'
@@ -304,6 +499,10 @@ def sortMovies(m1, m2):
     # Ok, these movies have the same production year...
     m1t = m1.get('canonical title', _last)
     m2t = m2.get('canonical title', _last)
+    # It should works also with normal dictionaries (returned from searches).
+    if m1t is _last and m2t is _last:
+        m1t = m1.get('title', _last)
+        m2t = m2.get('title', _last)
     if m1t < m2t: return -1
     if m1t > m2t: return 1
     # Ok, these movies have the same title...
@@ -324,6 +523,9 @@ def sortPeople(p1, p2):
     if p1b < p2b: return -1
     p1n = p1.get('canonical name', _last)
     p2n = p2.get('canonical name', _last)
+    if p1n is _last and p2n is _last:
+        p1n = p1.get('name', _last)
+        p2n = p2.get('name', _last)
     if p1n > p2n: return 1
     if p1n < p2n: return -1
     p1i = p1.get('imdbIndex', _last)
@@ -357,35 +559,57 @@ def modClearRefs(s, titlesRefs, namesRefs):
     s = modClearTitleRefs(s, {}, {})
     return modClearNameRefs(s, {}, {})
 
-_stypes = (type(u''), type(''))
-_ltype = type([])
-_dtype = type({})
-_todescend = (_ltype, _dtype)
 
 def modifyStrings(o, modFunct, titlesRefs, namesRefs):
     """Modify a string (or string values in a dictionary or strings
     in a list), using the provided modFunct function and titlesRefs
     and namesRefs references dictionaries."""
-    to = type(o)
-    if to in _stypes:
+    if isinstance(o, (UnicodeType, StringType)):
         return modFunct(o, titlesRefs, namesRefs)
-    elif to in _todescend:
+    elif isinstance(o, (ListType, TupleType)):
         _stillorig = 1
-        if to is _ltype: keys = xrange(len(o))
+        if isinstance(o, ListType): keys = xrange(len(o))
         else: keys = o.keys()
         for i in keys:
             v = o[i]
-            tv = type(v)
-            if tv in _stypes:
+            if isinstance(v, (UnicodeType, StringType)):
                 if _stillorig:
                     o = copy(o)
                     _stillorig = 0
                 o[i] = modFunct(v, titlesRefs, namesRefs)
-            elif tv in _todescend:
+            elif isinstance(v, (ListType, TupleType)):
                 modifyStrings(o[i], modFunct, titlesRefs, namesRefs)
     return o
 
 
+def flatten(seq, to_descend=(ListType, DictType, TupleType),
+            yieldDictKeys=0, scalar=None):
+    """Iterate over nested lists and dictionaries; to_descend is a type
+    of a tuple of types to be considered non-scalar; if yieldDictKeys is
+    true, also dictionaries' keys are yielded; if scalar is not None, only
+    items of the given type(s) are yielded."""
+    if not isinstance(seq, to_descend):
+        if scalar is None or isinstance(seq, scalar):
+            yield seq
+    else:
+        if isinstance(seq, DictType):
+            if yieldDictKeys:
+                # Yield also the keys of the dictionary.
+                for key in seq.iterkeys():
+                    for k in flatten(key, to_descend=to_descend,
+                                yieldDictKeys=yieldDictKeys, scalar=scalar):
+                        yield k
+            for value in seq.itervalues():
+                for v in flatten(value, to_descend=to_descend,
+                                yieldDictKeys=yieldDictKeys, scalar=scalar):
+                    yield v
+        else:
+            for item in seq:
+                for i in flatten(item, to_descend=to_descend,
+                                yieldDictKeys=yieldDictKeys, scalar=scalar):
+                    yield i
+
+
 class _Container:
     """Base class for Movie and Person classes."""
      # The default sets of information retrieved.
@@ -474,7 +698,7 @@ class _Container:
     def has_current_info(self, val):
         """Return true if the given set of information is in the list."""
         return val in self.current_info
-    
+
     def set_mod_funct(self, modFunct):
         """Set the fuction used to modify the strings."""
         if modFunct is None: modFunct = modClearRefs
@@ -483,7 +707,7 @@ class _Container:
     def update_titlesRefs(self, titlesRefs):
         """Update the dictionary with the references to movies."""
         self.titlesRefs.update(titlesRefs)
-    
+
     def get_titlesRefs(self):
         """Return the dictionary with the references to movies."""
         return self.titlesRefs
@@ -506,6 +730,10 @@ class _Container:
         else:
             self.data = data
 
+    def getID(self):
+        """Return movie or person ID."""
+        raise NotImplementedError, 'override this method'
+
     def __cmp__(self, other):
         """Compare two Movie or Person objects."""
         # XXX: only check the title/name and the movieID/personID?
@@ -516,6 +744,21 @@ class _Container:
             return 0
         return 1
 
+    def __hash__(self):
+        """Hash for this object."""
+        # XXX: does it always work correctly?
+        theID = self.getID()
+        if theID is not None and self.accessSystem not in ('UNKNOWN', None):
+            s4h = '%s:%s' % (self.accessSystem, theID)
+        else:
+            s4h = repr(self)
+        return hash(s4h)
+
+    def isSame(self, other):
+        if not isinstance(other, self.__class__): return 0
+        if hash(self) == hash(other): return 1
+        return 0
+
     def __len__(self):
         return len(self.data)
 
@@ -596,7 +839,15 @@ class _Container:
 
     def popitem(self):
         return self.data.popitem()
-    
+
+    def __repr__(self):
+        """String representation of an object."""
+        raise NotImplementedError, 'override this method'
+
+    def __str__(self):
+        """Movie title or person name."""
+        raise NotImplementedError, 'override this method'
+
     def __contains__(self, key):
         raise NotImplementedError, 'override this method'
 
